@@ -1,5 +1,5 @@
 /* ========================================================================================== */
-/* FMOD Ex - C++ header file. Copyright (c), Firelight Technologies Pty, Ltd. 2004.           */
+/* FMOD Ex - C++ header file. Copyright (c), Firelight Technologies Pty, Ltd. 2004-2008.      */
 /*                                                                                            */
 /* Use this header in conjunction with fmod.h (which contains all the constants / callbacks)  */
 /* to develop using C++ classes.                                                              */
@@ -14,7 +14,6 @@
     Constant and defines
 */
 
-
 /*
     FMOD Namespace
 */
@@ -24,8 +23,10 @@ namespace FMOD
     class Sound;
     class Channel;
     class ChannelGroup;
+    class SoundGroup;
     class Reverb;
     class DSP;
+    class DSPConnection;
     class Geometry;
 
     /*
@@ -61,7 +62,7 @@ namespace FMOD
         FMOD_RESULT F_API setOutput              (FMOD_OUTPUTTYPE output);
         FMOD_RESULT F_API getOutput              (FMOD_OUTPUTTYPE *output);
         FMOD_RESULT F_API getNumDrivers          (int *numdrivers);
-        FMOD_RESULT F_API getDriverName          (int id, char *name, int namelen);
+        FMOD_RESULT F_API getDriverInfo          (int id, char *name, int namelen, FMOD_GUID *guid);
         FMOD_RESULT F_API getDriverCaps          (int id, FMOD_CAPS *caps, int *minfrequency, int *maxfrequency, FMOD_SPEAKERMODE *controlpanelspeakermode);
         FMOD_RESULT F_API setDriver              (int driver);
         FMOD_RESULT F_API getDriver              (int *driver);
@@ -78,6 +79,7 @@ namespace FMOD
         FMOD_RESULT F_API getAdvancedSettings    (FMOD_ADVANCEDSETTINGS *settings);
         FMOD_RESULT F_API setSpeakerMode         (FMOD_SPEAKERMODE speakermode);
         FMOD_RESULT F_API getSpeakerMode         (FMOD_SPEAKERMODE *speakermode);
+        FMOD_RESULT F_API setCallback            (FMOD_SYSTEM_CALLBACKTYPE type, FMOD_SYSTEM_CALLBACK callback);
                                                 
         // Plug-in support                       
         FMOD_RESULT F_API setPluginPath          (const char *path);
@@ -102,9 +104,10 @@ namespace FMOD
         FMOD_RESULT F_API get3DNumListeners      (int *numlisteners);
         FMOD_RESULT F_API set3DListenerAttributes(int listener, const FMOD_VECTOR *pos, const FMOD_VECTOR *vel, const FMOD_VECTOR *forward, const FMOD_VECTOR *up);
         FMOD_RESULT F_API get3DListenerAttributes(int listener, FMOD_VECTOR *pos, FMOD_VECTOR *vel, FMOD_VECTOR *forward, FMOD_VECTOR *up);
+        FMOD_RESULT F_API set3DRolloffCallback   (FMOD_3D_ROLLOFFCALLBACK callback);
+        FMOD_RESULT F_API set3DSpeakerPosition   (FMOD_SPEAKER speaker, float x, float y, bool active);
+        FMOD_RESULT F_API get3DSpeakerPosition   (FMOD_SPEAKER speaker, float *x, float *y, bool *active);
 
-        FMOD_RESULT F_API setSpeakerPosition     (FMOD_SPEAKER speaker, float x, float y);
-        FMOD_RESULT F_API getSpeakerPosition     (FMOD_SPEAKER speaker, float *x, float *y);
         FMOD_RESULT F_API setStreamBufferSize    (unsigned int filebuffersize, FMOD_TIMEUNIT filebuffersizetype);
         FMOD_RESULT F_API getStreamBufferSize    (unsigned int *filebuffersize, FMOD_TIMEUNIT *filebuffersizetype);
                                                 
@@ -127,27 +130,34 @@ namespace FMOD
         FMOD_RESULT F_API createDSPByType        (FMOD_DSP_TYPE type, DSP **dsp);
         FMOD_RESULT F_API createDSPByIndex       (int index, DSP **dsp);
         FMOD_RESULT F_API createChannelGroup     (const char *name, ChannelGroup **channelgroup);
+        FMOD_RESULT F_API createSoundGroup       (const char *name, SoundGroup **soundgroup);
+        FMOD_RESULT F_API createReverb           (Reverb **reverb); 
                                                 
         FMOD_RESULT F_API playSound              (FMOD_CHANNELINDEX channelid, Sound *sound, bool paused, Channel **channel);
         FMOD_RESULT F_API playDSP                (FMOD_CHANNELINDEX channelid, DSP *dsp, bool paused, Channel **channel);
         FMOD_RESULT F_API getChannel             (int channelid, Channel **channel);
         FMOD_RESULT F_API getMasterChannelGroup  (ChannelGroup **channelgroup);
-                                               
+        FMOD_RESULT F_API getMasterSoundGroup    (SoundGroup **soundgroup);
+                                              
         // Reverb API                           
         FMOD_RESULT F_API setReverbProperties    (const FMOD_REVERB_PROPERTIES *prop);
         FMOD_RESULT F_API getReverbProperties    (FMOD_REVERB_PROPERTIES *prop);
+        FMOD_RESULT F_API setReverbAmbientProperties(FMOD_REVERB_PROPERTIES *prop);
+        FMOD_RESULT F_API getReverbAmbientProperties(FMOD_REVERB_PROPERTIES *prop);
                                                  
         // System level DSP access.
         FMOD_RESULT F_API getDSPHead             (DSP **dsp);
-        FMOD_RESULT F_API addDSP                 (DSP *dsp);
+        FMOD_RESULT F_API addDSP                 (DSP *dsp, DSPConnection **connection);
         FMOD_RESULT F_API lockDSP                ();
         FMOD_RESULT F_API unlockDSP              ();
+        FMOD_RESULT F_API getDSPClock            (unsigned int *hi, unsigned int *lo);
                                                
         // Recording API.
         FMOD_RESULT F_API setRecordDriver        (int driver);
         FMOD_RESULT F_API getRecordDriver        (int *driver);
         FMOD_RESULT F_API getRecordNumDrivers    (int *numdrivers);
-        FMOD_RESULT F_API getRecordDriverName    (int id, char *name, int namelen);
+        FMOD_RESULT F_API getRecordDriverInfo    (int id, char *name, int namelen, FMOD_GUID *guid);
+        FMOD_RESULT F_API getRecordDriverCaps    (int id, FMOD_CAPS *caps, int *minfrequency, int *maxfrequency);
         FMOD_RESULT F_API getRecordPosition      (unsigned int *position);  
 
         FMOD_RESULT F_API recordStart            (Sound *sound, bool loop);
@@ -211,6 +221,9 @@ namespace FMOD
         FMOD_RESULT F_API readData               (void *buffer, unsigned int lenbytes, unsigned int *read);
         FMOD_RESULT F_API seekData               (unsigned int pcm);
 
+        FMOD_RESULT F_API setSoundGroup          (SoundGroup *soundgroup);
+        FMOD_RESULT F_API getSoundGroup          (SoundGroup **soundgroup);
+
         // Synchronization point API.  These points can come from markers embedded in wav files, and can also generate channel callbacks.        
         FMOD_RESULT F_API getNumSyncPoints       (int *numsyncpoints);
         FMOD_RESULT F_API getSyncPoint           (int index, FMOD_SYNCPOINT **point);
@@ -225,6 +238,11 @@ namespace FMOD
         FMOD_RESULT F_API getLoopCount           (int *loopcount);
         FMOD_RESULT F_API setLoopPoints          (unsigned int loopstart, FMOD_TIMEUNIT loopstarttype, unsigned int loopend, FMOD_TIMEUNIT loopendtype);
         FMOD_RESULT F_API getLoopPoints          (unsigned int *loopstart, FMOD_TIMEUNIT loopstarttype, unsigned int *loopend, FMOD_TIMEUNIT loopendtype);
+
+        // For MOD/S3M/XM/IT/MID sequenced formats only.
+        FMOD_RESULT F_API getMusicNumChannels    (int *numchannels);
+        FMOD_RESULT F_API setMusicChannelVolume  (int channel, float volume);
+        FMOD_RESULT F_API getMusicChannelVolume  (int channel, float *volume);
                             
         // Userdata set/get.
         FMOD_RESULT F_API setUserData            (void *userdata);
@@ -253,12 +271,14 @@ namespace FMOD
         FMOD_RESULT F_API getFrequency           (float *frequency);
         FMOD_RESULT F_API setPan                 (float pan);
         FMOD_RESULT F_API getPan                 (float *pan);
-        FMOD_RESULT F_API setDelay               (unsigned int startdelay, unsigned int enddelay);
-        FMOD_RESULT F_API getDelay               (unsigned int *startdelay, unsigned int *enddelay);
+        FMOD_RESULT F_API setDelay               (FMOD_DELAYTYPE delaytype, unsigned int delayhi, unsigned int delaylo);
+        FMOD_RESULT F_API getDelay               (FMOD_DELAYTYPE delaytype, unsigned int *delayhi, unsigned int *delaylo);
         FMOD_RESULT F_API setSpeakerMix          (float frontleft, float frontright, float center, float lfe, float backleft, float backright, float sideleft, float sideright);
         FMOD_RESULT F_API getSpeakerMix          (float *frontleft, float *frontright, float *center, float *lfe, float *backleft, float *backright, float *sideleft, float *sideright);
         FMOD_RESULT F_API setSpeakerLevels       (FMOD_SPEAKER speaker, float *levels, int numlevels);
         FMOD_RESULT F_API getSpeakerLevels       (FMOD_SPEAKER speaker, float *levels, int numlevels);
+        FMOD_RESULT F_API setInputChannelMix     (float *levels, int numlevels);
+        FMOD_RESULT F_API getInputChannelMix     (float *levels, int numlevels);
         FMOD_RESULT F_API setMute                (bool mute);
         FMOD_RESULT F_API getMute                (bool *mute);
         FMOD_RESULT F_API setPriority            (int priority);
@@ -267,6 +287,7 @@ namespace FMOD
         FMOD_RESULT F_API getPosition            (unsigned int *position, FMOD_TIMEUNIT postype);
         FMOD_RESULT F_API setReverbProperties    (const FMOD_REVERB_CHANNELPROPERTIES *prop);
         FMOD_RESULT F_API getReverbProperties    (FMOD_REVERB_CHANNELPROPERTIES *prop);
+
         FMOD_RESULT F_API setChannelGroup        (ChannelGroup *channelgroup);
         FMOD_RESULT F_API getChannelGroup        (ChannelGroup **channelgroup);
         FMOD_RESULT F_API setCallback            (FMOD_CHANNEL_CALLBACKTYPE type, FMOD_CHANNEL_CALLBACK callback, int command);
@@ -293,7 +314,7 @@ namespace FMOD
 
         // DSP functionality only for channels playing sounds created with FMOD_SOFTWARE.
         FMOD_RESULT F_API getDSPHead             (DSP **dsp);
-        FMOD_RESULT F_API addDSP                 (DSP *dsp);
+        FMOD_RESULT F_API addDSP                 (DSP *dsp, DSPConnection **connection);
 
         // Information only functions.
         FMOD_RESULT F_API isPlaying              (bool *isplaying);
@@ -336,6 +357,8 @@ namespace FMOD
         FMOD_RESULT F_API getVolume               (float *volume);
         FMOD_RESULT F_API setPitch                (float pitch);
         FMOD_RESULT F_API getPitch                (float *pitch);
+        FMOD_RESULT F_API set3DOcclusion          (float directocclusion, float reverbocclusion);
+        FMOD_RESULT F_API get3DOcclusion          (float *directocclusion, float *reverbocclusion);
         FMOD_RESULT F_API setPaused               (bool paused);
         FMOD_RESULT F_API getPaused               (bool *paused);
         FMOD_RESULT F_API setMute                 (bool mute);
@@ -343,11 +366,9 @@ namespace FMOD
 
         // Channelgroup override values.  (recursively overwrites whatever settings the channels had)
         FMOD_RESULT F_API stop                    ();
-        FMOD_RESULT F_API overridePaused          (bool paused);
         FMOD_RESULT F_API overrideVolume          (float volume);
         FMOD_RESULT F_API overrideFrequency       (float frequency);
         FMOD_RESULT F_API overridePan             (float pan);
-        FMOD_RESULT F_API overrideMute            (bool mute);
         FMOD_RESULT F_API overrideReverbProperties(const FMOD_REVERB_CHANNELPROPERTIES *prop);
         FMOD_RESULT F_API override3DAttributes    (const FMOD_VECTOR *pos, const FMOD_VECTOR *vel);
         FMOD_RESULT F_API overrideSpeakerMix      (float frontleft, float frontright, float center, float lfe, float backleft, float backright, float sideleft, float sideright);
@@ -356,10 +377,11 @@ namespace FMOD
         FMOD_RESULT F_API addGroup                (ChannelGroup *group);
         FMOD_RESULT F_API getNumGroups            (int *numgroups);
         FMOD_RESULT F_API getGroup                (int index, ChannelGroup **group);
+        FMOD_RESULT F_API getParentGroup          (ChannelGroup **group);
 
         // DSP functionality only for channel groups playing sounds created with FMOD_SOFTWARE.
         FMOD_RESULT F_API getDSPHead              (DSP **dsp);
-        FMOD_RESULT F_API addDSP                  (DSP *dsp);
+        FMOD_RESULT F_API addDSP                  (DSP *dsp, DSPConnection **connection);
 
         // Information only functions.
         FMOD_RESULT F_API getName                 (char *name, int namelen);
@@ -373,6 +395,42 @@ namespace FMOD
         FMOD_RESULT F_API getUserData             (void **userdata);
     };
 
+    /*
+        'SoundGroup' API
+    */
+    class SoundGroup
+    {
+      private:
+
+        SoundGroup();       /* Constructor made private so user cannot statically instance a SoundGroup class.  
+                               Appropriate SoundGroup creation or retrieval function must be used. */
+      public:
+
+        FMOD_RESULT F_API release                ();
+        FMOD_RESULT F_API getSystemObject        (System **system);
+
+        // SoundGroup control functions.
+        FMOD_RESULT F_API setMaxAudible          (int maxaudible);
+        FMOD_RESULT F_API getMaxAudible          (int *maxaudible);
+        FMOD_RESULT F_API setMaxAudibleBehavior  (FMOD_SOUNDGROUP_BEHAVIOR behavior);
+        FMOD_RESULT F_API getMaxAudibleBehavior  (FMOD_SOUNDGROUP_BEHAVIOR *behavior);
+        FMOD_RESULT F_API setMuteFadeSpeed       (float speed);
+        FMOD_RESULT F_API getMuteFadeSpeed       (float *speed);
+        FMOD_RESULT F_API setVolume              (float volume);
+        FMOD_RESULT F_API getVolume              (float *volume);
+        FMOD_RESULT F_API stop                   ();
+
+        // Information only functions.
+        FMOD_RESULT F_API getName                (char *name, int namelen);
+        FMOD_RESULT F_API getNumSounds           (int *numsounds);
+        FMOD_RESULT F_API getSound               (int index, Sound **sound);
+        FMOD_RESULT F_API getNumPlaying          (int *numplaying);
+
+        // Userdata set/get.
+        FMOD_RESULT F_API setUserData            (void *userdata);
+        FMOD_RESULT F_API getUserData            (void **userdata);
+    };
+
     /*
         'DSP' API
     */
@@ -388,22 +446,14 @@ namespace FMOD
         FMOD_RESULT F_API getSystemObject        (System **system);
 
         // Connection / disconnection / input and output enumeration.
-        FMOD_RESULT F_API addInput               (DSP *target);
+        FMOD_RESULT F_API addInput               (DSP *target, DSPConnection **connection);
         FMOD_RESULT F_API disconnectFrom         (DSP *target);
         FMOD_RESULT F_API disconnectAll          (bool inputs, bool outputs);
         FMOD_RESULT F_API remove                 ();
         FMOD_RESULT F_API getNumInputs           (int *numinputs);
         FMOD_RESULT F_API getNumOutputs          (int *numoutputs);
-        FMOD_RESULT F_API getInput               (int index, DSP **input);
-        FMOD_RESULT F_API getOutput              (int index, DSP **output);
-        FMOD_RESULT F_API setInputMix            (int index, float volume);
-        FMOD_RESULT F_API getInputMix            (int index, float *volume);
-        FMOD_RESULT F_API setInputLevels         (int index, FMOD_SPEAKER speaker, float *levels, int numlevels);
-        FMOD_RESULT F_API getInputLevels         (int index, FMOD_SPEAKER speaker, float *levels, int numlevels);
-        FMOD_RESULT F_API setOutputMix           (int index, float volume);
-        FMOD_RESULT F_API getOutputMix           (int index, float *volume);
-        FMOD_RESULT F_API setOutputLevels        (int index, FMOD_SPEAKER speaker, float *levels, int numlevels);
-        FMOD_RESULT F_API getOutputLevels        (int index, FMOD_SPEAKER speaker, float *levels, int numlevels);
+        FMOD_RESULT F_API getInput               (int index, DSP **input, DSPConnection **inputconnection);
+        FMOD_RESULT F_API getOutput              (int index, DSP **output, DSPConnection **outputconnection);
 
         // DSP unit control.
         FMOD_RESULT F_API setActive              (bool active);
@@ -431,6 +481,31 @@ namespace FMOD
     };
 
 
+    /*
+        'DSPConnection' API
+    */
+    class DSPConnection
+    {
+      private:
+
+        DSPConnection();    /* Constructor made private so user cannot statically instance a DSPConnection class.  
+                               Appropriate DSPConnection creation or retrieval function must be used. */
+
+      public:
+
+        FMOD_RESULT F_API getInput              (DSP **input);
+        FMOD_RESULT F_API getOutput             (DSP **output);
+        FMOD_RESULT F_API setMix                (float volume);
+        FMOD_RESULT F_API getMix                (float *volume);
+        FMOD_RESULT F_API setLevels             (FMOD_SPEAKER speaker, float *levels, int numlevels);
+        FMOD_RESULT F_API getLevels             (FMOD_SPEAKER speaker, float *levels, int numlevels);
+
+        // Userdata set/get.
+        FMOD_RESULT F_API setUserData           (void *userdata);
+        FMOD_RESULT F_API getUserData           (void **userdata);
+    };
+
+
     /*
         'Geometry' API
     */
@@ -445,9 +520,7 @@ namespace FMOD
 
         FMOD_RESULT F_API release                ();
 
-        /*
-            Polygon manipulation.
-        */
+        // Polygon manipulation.
         FMOD_RESULT F_API addPolygon             (float directocclusion, float reverbocclusion, bool doublesided, int numvertices, const FMOD_VECTOR *vertices, int *polygonindex);
         FMOD_RESULT F_API getNumPolygons         (int *numpolygons); 
         FMOD_RESULT F_API getMaxPolygons         (int *maxpolygons, int *maxvertices);
@@ -457,9 +530,7 @@ namespace FMOD
         FMOD_RESULT F_API setPolygonAttributes   (int index, float directocclusion, float reverbocclusion, bool doublesided); 
         FMOD_RESULT F_API getPolygonAttributes   (int index, float *directocclusion, float *reverbocclusion, bool *doublesided); 
 
-        /*
-            Object manipulation.
-        */
+        // Object manipulation.
         FMOD_RESULT F_API setActive              (bool active);                                                 
         FMOD_RESULT F_API getActive              (bool *active);                                                 
         FMOD_RESULT F_API setRotation            (const FMOD_VECTOR *forward, const FMOD_VECTOR *up);
@@ -476,6 +547,32 @@ namespace FMOD
     };
 
 
-};
+    /*
+        'Reverb' API
+    */
+    class Reverb
+    {
+      private:
+
+        Reverb();    /*  Constructor made private so user cannot statically instance a Reverb class.  
+                         Appropriate Reverb creation or retrieval function must be used. */
+
+      public:    
+
+        FMOD_RESULT F_API release                ();
+
+        // Reverb manipulation.
+        FMOD_RESULT F_API set3DAttributes        (const FMOD_VECTOR *position, float mindistance, float maxdistance);
+        FMOD_RESULT F_API get3DAttributes        (FMOD_VECTOR *position, float *mindistance,float *maxdistance);
+        FMOD_RESULT F_API setProperties          (const FMOD_REVERB_PROPERTIES *properties);
+        FMOD_RESULT F_API getProperties          (FMOD_REVERB_PROPERTIES *properties);
+        FMOD_RESULT F_API setActive              (bool active);
+        FMOD_RESULT F_API getActive              (bool *active);
+
+        // Userdata set/get.
+        FMOD_RESULT F_API setUserData            (void *userdata);
+        FMOD_RESULT F_API getUserData            (void **userdata);    
+    };
+}
 
 #endif
